import os
import mimetypes
import json

from flask import Flask, request, Response, jsonify, send_file, abort, render_template_string
from flask_cors import CORS
import yaml
from jsonschema import validate, ValidationError

from riseml.config_parser import parse_file


template = '''<!DOCTYPE html>
<html>
<body>
<p>This host is running an API endpoint at <code>/predict</code>.</p>
<p>Input</p>
<ul>
{% if deploy.input %}
{% for name, schema_name in deploy.input.items() %}
<li>{{ name }}: <code>{{ schema_name }}</code></li>
{% endfor %}
{% endif %}
</ul>
<p>Output</p>
<ul>
{% if deploy.output %}
{% for name, schema_name in deploy.output.items() %}
<li>{{ name }}: <code>{{ schema_name }}</code></li>
{% endfor %}
{% endif %}
</ul>
</body>
</html>'''

def serve(func,
          host=os.environ.get('HOST', '0.0.0.0'),
          port=os.environ.get('PORT')):

    def get_mimetype(value):
        if value:
            if value in mimetypes.types_map.values():
                return value, None
            return 'application/vnd.riseml+%s' % value, value
        return 'application/octet-stream', None

    app = Flask(__name__)
    CORS(app, max_age=3600)
    config = parse_file('riseml.yml')

    schemas = {}
    if config and config.deploy and config.deploy.output and config.deploy.output:
        for name, schema_name in config.deploy.output.items():
            if get_mimetype(schema_name)[1]:
                root = os.path.abspath(os.path.dirname(__file__))
                loc = os.path.join(root, 'schemas', schema_name + '.yml')
                if not os.path.isfile(loc):
                    raise Exception('schema %s not found' % schema_name)
                with open(loc, 'rb') as f:
                    schemas[schema_name] = yaml.load(f.read())

    def _validate(obj, schema_name):
        if schema_name in schemas:
            validate(obj, schemas[schema_name])
            return json.dumps(obj)
        return obj

    @app.route('/')
    def _root():
        return render_template_string(template,
            deploy=config.deploy)

    @app.route('/predict', methods=['POST'])
    def _predict():
        for name, schema_name in config.deploy.output.items():
            try:
                return Response(
                    _validate(func(request.files[name].read()), schema_name),
                    mimetype=get_mimetype(schema_name)[0])
            except ValidationError as e:
                return jsonify({'error': 'invalid %s: %s' % (schema_name, e.message)})

    @app.route('/config', methods=['GET'])
    def _config():
        return jsonify(config.to_dict())

    @app.route('/samples/<path:path>', methods=['GET'])
    def _samples(path):
        if (config.deploy and config.deploy.demo and
            config.deploy.demo.samples and
            path in config.deploy.demo.samples):

            full_path = os.path.join(os.getcwd(), path)
            if not os.path.exists(full_path):
                abort(404)
            with open(full_path) as f:
                return Response(
                    func(f.read()),
                    mimetype='image/jpeg')

        abort(404)

    app.run(host=host, port=port, threaded=False)



# coding: utf-8

"""
    RiseML API


    OpenAPI spec version: 1.1.0
    Contact: contact@riseml.com
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

from pprint import pformat
from six import iteritems
import re


class Repository(object):
    """
    NOTE: This class is auto generated by the swagger code generator program.
    Do not edit the class manually.
    """
    def __init__(self, id=None, name=None, description=None, service_name=None, changeset=None, job=None, username=None, full_name=None):
        """
        Repository - a model defined in Swagger

        :param dict swaggerTypes: The key is attribute name
                                  and the value is attribute type.
        :param dict attributeMap: The key is attribute name
                                  and the value is json key in definition.
        """
        self.swagger_types = {
            'id': 'str',
            'name': 'str',
            'description': 'str',
            'service_name': 'str',
            'changeset': 'Changeset',
            'job': 'Job',
            'username': 'str',
            'full_name': 'str'
        }

        self.attribute_map = {
            'id': 'id',
            'name': 'name',
            'description': 'description',
            'service_name': 'service_name',
            'changeset': 'changeset',
            'job': 'job',
            'username': 'username',
            'full_name': 'full_name'
        }

        self._id = id
        self._name = name
        self._description = description
        self._service_name = service_name
        self._changeset = changeset
        self._job = job
        self._username = username
        self._full_name = full_name


    @property
    def id(self):
        """
        Gets the id of this Repository.


        :return: The id of this Repository.
        :rtype: str
        """
        return self._id

    @id.setter
    def id(self, id):
        """
        Sets the id of this Repository.


        :param id: The id of this Repository.
        :type: str
        """

        self._id = id

    @property
    def name(self):
        """
        Gets the name of this Repository.


        :return: The name of this Repository.
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """
        Sets the name of this Repository.


        :param name: The name of this Repository.
        :type: str
        """

        self._name = name

    @property
    def description(self):
        """
        Gets the description of this Repository.


        :return: The description of this Repository.
        :rtype: str
        """
        return self._description

    @description.setter
    def description(self, description):
        """
        Sets the description of this Repository.


        :param description: The description of this Repository.
        :type: str
        """

        self._description = description

    @property
    def service_name(self):
        """
        Gets the service_name of this Repository.


        :return: The service_name of this Repository.
        :rtype: str
        """
        return self._service_name

    @service_name.setter
    def service_name(self, service_name):
        """
        Sets the service_name of this Repository.


        :param service_name: The service_name of this Repository.
        :type: str
        """

        self._service_name = service_name

    @property
    def changeset(self):
        """
        Gets the changeset of this Repository.


        :return: The changeset of this Repository.
        :rtype: Changeset
        """
        return self._changeset

    @changeset.setter
    def changeset(self, changeset):
        """
        Sets the changeset of this Repository.


        :param changeset: The changeset of this Repository.
        :type: Changeset
        """

        self._changeset = changeset

    @property
    def job(self):
        """
        Gets the job of this Repository.


        :return: The job of this Repository.
        :rtype: Job
        """
        return self._job

    @job.setter
    def job(self, job):
        """
        Sets the job of this Repository.


        :param job: The job of this Repository.
        :type: Job
        """

        self._job = job

    @property
    def username(self):
        """
        Gets the username of this Repository.


        :return: The username of this Repository.
        :rtype: str
        """
        return self._username

    @username.setter
    def username(self, username):
        """
        Sets the username of this Repository.


        :param username: The username of this Repository.
        :type: str
        """

        self._username = username

    @property
    def full_name(self):
        """
        Gets the full_name of this Repository.


        :return: The full_name of this Repository.
        :rtype: str
        """
        return self._full_name

    @full_name.setter
    def full_name(self, full_name):
        """
        Sets the full_name of this Repository.


        :param full_name: The full_name of this Repository.
        :type: str
        """

        self._full_name = full_name

    def to_dict(self):
        """
        Returns the model properties as a dict
        """
        result = {}

        for attr, _ in iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """
        Returns the string representation of the model
        """
        return pformat(self.to_dict())

    def __repr__(self):
        """
        For `print` and `pprint`
        """
        return self.to_str()

    def __eq__(self, other):
        """
        Returns true if both objects are equal
        """
        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """
        Returns true if both objects are not equal
        """
        return not self == other
